/*
 *  videoWindow.c
 *  seeSaw
 *
 *  Created by Daniel Heckenberg.
 *  Copyright (c) 2004 Daniel Heckenberg. All rights reserved.
 *  (danielh.seeSaw<at>cse<dot>unsw<dot>edu<dot>au)
 *  
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the right to use, copy, modify, merge, publish, communicate, sublicence, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included 
 * in all copies or substantial portions of the Software.
 * 
 * TO THE EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED 
 * "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT 
 * NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NON-INFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 */

#include <GLUT/glut.h>
#include <OpenGL/glext.h>
#include <OpenGL/gl.h>
#include <OpenGL/glu.h>

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#include "SSUtility.h"
#include "vdigGrab.h"

#include "videoWindow.h"

#pragma mark ---- Video Window constants ----

const int gMilliSecPerTimerMin = 20;
const int gMilliSecPerTimerMax = 100;

#pragma mark ---- Video Window data ----

// Video Window data
struct tagVideoWindow
{
	// GLUT window info
	int			winID;  
	int			width;
	int			height;
	int			isFullScreen;
	int			widthLast;
	int			heightLast;
	int			xPosLast;
	int			yPosLast;
	long		milliSecPerTimer;
	
	// vdig grab data
	VdigGrab*   pVdg;
	long		milliSecPerFrame;
	Fixed		frameRate;
	long		bytesPerSecond;
	ImageDescriptionHandle vdImageDesc;
	
	// Destination port (GWorld)
	CGrafPtr	dstPort;
	Rect		dstPortBounds;
	Rect		dstDisplayBounds;
	void*		pDstData;
	long		dstDataSize;

	// GL information
	int			useClientStorage;
	int			useTextureRectangle;
	int			useTextureRange;

	// texture information
	GLenum		texEnum;
	GLint		texFilter;	
	GLuint		texID;
	GLint		texWidth;
	GLint		texHeight;
	int			texIsFirstUploaded;
};

#pragma mark ---- Video Window forward declarations ----

void
videoInitGL(VideoWindow* pWnd);

void
videoInitTexture(VideoWindow* pWnd, int width, int height);

OSErr
videoInitOffscreen (VideoWindow* pWnd, int width, int height);

void 
videoReshape(VideoWindow* pWnd, int w, int h);    

void 
videoReshapeDispatch(int w, int h);

void 
videoDisplay(VideoWindow* pWnd);   

void 
videoDisplayDispatch(void);

void 
videoUpdateFrame(VideoWindow* pWnd, void* pData, long dataSize);

void
videoTimer(VideoWindow* pWnd);

void 
videoTimerDispatch(int winID);

void 
videoKeyboard(VideoWindow* pWnd, unsigned char key, int x, int y );

void 
videoKeyboardDispatch ( unsigned char key, int x, int y );

void
videoMouse(VideoWindow* pWnd, int button, int state, int x, int y);

void 
videoMouseDispatch (int button, int state, int x, int y);

OSErr
videoSetWindowTitle (VideoWindow* pWnd);

OSErr
videoSetTimerPeriodFromVdig(VideoWindow* pWnd);

void
videoSetDisplayBoundsToCaptureBounds(VideoWindow* pWnd);

int
videoGetGLUnpackAlignment(VideoWindow* pWnd);

int 
videoGetGLUnpackRowLength(VideoWindow* pWnd);

void
videoGetTexCoords(VideoWindow* pWnd, float* aCoords);

#pragma mark ---- Video Window functions ----

VideoWindow*
videoNewWindow() 
{
	VideoWindow* pWnd = NULL;

	if (!(pWnd = malloc(sizeof(VideoWindow))))
	{
		printError("videoNewWindow: could not allocate VideoWindow structure\n");	
		goto endFunc;
	}

	// zero initialize the structure
	memset(pWnd, 0, sizeof(VideoWindow));

	// Create the vdig grabber
	OSErr err;
	
	if(!(pWnd->pVdg = vdgNew()))
	{
		printError("vdgNew: failed to allocate\n");	
		goto endFunc;
	}

	if(err = vdgInit(pWnd->pVdg))
	{
		printError("vdgInit err=%d\n", err);
		goto endFunc;
	}

	if(err = vdgRequestSettings(pWnd->pVdg))
	{
		printError("vdgRequestSettings err=%d\n", err);
		goto endFunc;
	}
	
	if(err = vdgPreflightGrabbing(pWnd->pVdg))
	{
		printError("vdgPreflightGrabbing err=%d\n", err);
		goto endFunc;
	}

	// Set the timer frequency from the Vdig's Data Rate
	videoSetTimerPeriodFromVdig(pWnd);
	
    pWnd->vdImageDesc = (ImageDescriptionHandle)NewHandle(0);
	if (err = vdgGetImageDescription( pWnd->pVdg, 
									  pWnd->vdImageDesc))
	{
		printError("vdgGetImageDescription err=%d\n", err);
		goto endFunc;
	}

	// now initialize glut
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA); 
	glutInitWindowSize ((*pWnd->vdImageDesc)->width, (*pWnd->vdImageDesc)->height);
	pWnd->winID = glutCreateWindow("Video Window");

	// add to the window management list
	if (!addDataforID(pWnd->winID, pWnd))
	{
		printError("addDataforID could not add data\n", err);
	}

    videoInitGL(pWnd); // standard GL init

	glutReshapeFunc (videoReshapeDispatch);
	glutDisplayFunc (videoDisplayDispatch);
	glutKeyboardFunc (videoKeyboardDispatch);
	glutMouseFunc( videoMouseDispatch);

	videoSetWindowTitle(pWnd);

	videoInitTexture(   pWnd, 
						(*pWnd->vdImageDesc)->width, 
						(*pWnd->vdImageDesc)->height);

	// Now create the offscreen GWorld to match the texture
	if (err = videoInitOffscreen( pWnd, pWnd->texWidth, pWnd->texHeight))
	{
		printError("videoInitOffscreen err=%d\n", err);
		goto endFunc;
	}
	
	// Set the decompression destination to the offscreen GWorld
	if (err = vdgSetDestination(	pWnd->pVdg,
									pWnd->dstPort ))
	{
		printError("vdgGetImageDescription err=%d\n", err);
		goto endFunc;	
	}
	
	// Set the display bounds to match only the part of the texture
	// that corresponds to the video image
	videoSetDisplayBoundsToCaptureBounds(pWnd);
							
	if (err = vdgStartGrabbing(pWnd->pVdg))
	{
		printError("vdgStartGrabbing err=%d\n", err);
		goto endFunc;
	}

	// Do the first poll straight away
	glutTimerFunc(0, videoTimerDispatch, pWnd->winID);
	
	
endFunc:
	return pWnd;
}

void
videoInitGL(VideoWindow* pWnd)
{
	glShadeModel(GL_FLAT);					
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);				// black window background

	// Set up texture filtering
	pWnd->texFilter = GL_LINEAR;

	// Check extension support
	pWnd->useTextureRectangle = glutExtensionSupported("GL_EXT_texture_rectangle");
	pWnd->useClientStorage = glutExtensionSupported("GL_APPLE_client_storage");
	pWnd->useTextureRange = glutExtensionSupported("GL_APPLE_texture_range");
}

void
videoInitTexture(VideoWindow* pWnd, int width, int height)
{
	if (pWnd->useTextureRectangle)
	{
		pWnd->texEnum = GL_TEXTURE_RECTANGLE_EXT;
		pWnd->texWidth = width;
		pWnd->texHeight = height;
	} else
	{
		pWnd->texEnum = GL_TEXTURE_2D;
		pWnd->texWidth = ceilpow2i(width);
		pWnd->texHeight = ceilpow2i(height);
	}
	
	glEnable(pWnd->texEnum);
	glGenTextures (1, &pWnd->texID);
	glDisable(pWnd->texEnum);
}

OSErr
videoInitOffscreen (VideoWindow* pWnd, int width, int height)
{
	OSErr err;

	// create the offscreen target
	pWnd->dstPortBounds.left = 0;
	pWnd->dstPortBounds.right = width;
	pWnd->dstPortBounds.top = 0;
	pWnd->dstPortBounds.bottom = height;
	if (err = createOffscreenGWorld(	&pWnd->dstPort,
										k422YpCbCr8CodecType,
										&pWnd->dstPortBounds))
	{
		printError("createOffscreenGWorld err=%d\n", err);
		goto endFunc;	
	}
	
	// Get buffer from GWorld
	pWnd->pDstData = GetPixBaseAddr(GetGWorldPixMap(pWnd->dstPort));
	pWnd->dstDataSize = GetPixRowBytes(GetGWorldPixMap(pWnd->dstPort)) * (pWnd->dstPortBounds.bottom - pWnd->dstPortBounds.top); 
	pWnd->dstDisplayBounds = pWnd->dstPortBounds;

endFunc:
	return err;
}

void 
videoReshape(VideoWindow* pWnd, int w, int h)    
{
	glViewport(0, 0, w, h);
	
	// store new width and height for drawing
	pWnd->width = w;
	pWnd->height = h;
							
	// attempt to work around bug? in Mac OS X GLUT where window title is lost after
	// return from fullscreen
	videoSetWindowTitle(pWnd);					
}	

void 
videoReshapeDispatch(int w, int h)
{
	int winID;
	VideoWindow* pWnd;
	
	winID = glutGetWindow();
	if (pWnd = (VideoWindow*) getDataForID(winID))
		videoReshape(pWnd, w, h);
	else
		printError("videoReshapeDispatch: unknown winID: %d\n", winID);
}

void 
videoDisplay(VideoWindow* pWnd)   
{
	// set up the context for pixel-based coordinates
	const float minWinX = 0.0F;
	const float maxWinX = pWnd->width;
	const float minWinY = 0.0F;
	const float maxWinY = pWnd->height;

	float texCoords[4];
	videoGetTexCoords(pWnd, texCoords);
	
	glClearColor (1.0f, 0.0f, 1.0f, 1.0f);	// clear the surface
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// An orthographic view with 0,0 in the upper left hand corner.
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	
	glOrtho(minWinX, maxWinX, maxWinY, minWinY, -1.0, 1.0);
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();

	glDisable(GL_LIGHTING);
	glEnable(pWnd->texEnum);
	
	glColor3f (1.0, 1.0, 1.0); // no coloring
	
	glBegin(GL_QUADS);
		glTexCoord2f(texCoords[0], texCoords[2]);
		glVertex2f(minWinX, minWinY);
		glTexCoord2f(texCoords[0], texCoords[3]);		
		glVertex2f(minWinX, maxWinY);
		glTexCoord2f(texCoords[1], texCoords[3]);		
		glVertex2f(maxWinX, maxWinY);
		glTexCoord2f(texCoords[1], texCoords[2]);
		glVertex2f(maxWinX, minWinY);
	glEnd();
	
	//BEN
	//void* pDstData;
	//long dstDataSize;
	glBegin(GL_POINTS);
		glVertex2f(0.0,0.0);
	glEnd();
	
	//
	
	glDisable(pWnd->texEnum);

	glPopMatrix();

	glMatrixMode(GL_PROJECTION);
	glPopMatrix();

	glMatrixMode(GL_MODELVIEW);

	glutSwapBuffers ( );
}

void 
videoDisplayDispatch(void)
{
	int winID;
	VideoWindow* pWnd;
	
	winID = glutGetWindow();
	if (pWnd = (VideoWindow*) getDataForID(winID))
		videoDisplay(pWnd);
	else
		printError("videoDisplayDispatch: unknown winID: %d\n", winID);
}


void 
videoUpdateFrame(VideoWindow* pWnd, void* pData, long dataSize)
{
	if (!pWnd->texID)
	{
		printError("videoUpdateFrame: texture not initialized\n");
		return;
	}

	glBindTexture(pWnd->texEnum, pWnd->texID);

	if (!pWnd->texIsFirstUploaded)
	{
		if (pWnd->useTextureRange)
		{  
			glTextureRangeAPPLE(pWnd->texEnum, dataSize, pData);  
			glTexParameteri(pWnd->texEnum, GL_TEXTURE_STORAGE_HINT_APPLE , GL_STORAGE_SHARED_APPLE); 
		} 
		
		if (pWnd->useClientStorage)
		{   
			// Note that if we use client storage, we must keep the first buffer uploaded
			// valid for the lifetime of the texture
			glPixelStorei(GL_UNPACK_CLIENT_STORAGE_APPLE, 1);
		}

		glTexParameteri(pWnd->texEnum, GL_TEXTURE_MIN_FILTER, pWnd->texFilter);
		glTexParameteri(pWnd->texEnum, GL_TEXTURE_MAG_FILTER, pWnd->texFilter);
		glTexParameteri(pWnd->texEnum, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		glTexParameteri(pWnd->texEnum, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	
		// Set the unpack factors correctly in case we're using a subimage of
		// a power of two texture
		if (pWnd->useTextureRectangle)
		{
			glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
			glPixelStorei(GL_UNPACK_ROW_LENGTH, 0);			
		} else
		{
			glPixelStorei(GL_UNPACK_ALIGNMENT, videoGetGLUnpackAlignment(pWnd));
			glPixelStorei(GL_UNPACK_ROW_LENGTH, videoGetGLUnpackRowLength(pWnd));
		}
		
		// Upload the first image.
		glTexImage2D(   pWnd->texEnum,		// GLenum target,
						0,					// GLint level,
						GL_RGBA,			// GLint internalformat,
						pWnd->dstPortBounds.right-pWnd->dstPortBounds.left,				// GLsizei width,
						pWnd->dstPortBounds.bottom-pWnd->dstPortBounds.top,				// GLsizei height,
						0,					// GLint border,
						GL_YCBCR_422_APPLE, //GL_BGR_EXT,  //GL_BGRA_EXT, // GLenum format,
						GL_UNSIGNED_SHORT_8_8_REV_APPLE, //GL_BYTE,  //GL_UNSIGNED_INT_8_8_8_8_REV, //GLenum type
						pData );			//const GLvoid *pixels
		pWnd->texIsFirstUploaded = 1;
	} else
	{
		glTexSubImage2D(pWnd->texEnum,					// GLenum target,
						0,								// GLint level,
						pWnd->dstDisplayBounds.left,	// GLint xoffset
						pWnd->dstDisplayBounds.top,		// GLint yoffset
						pWnd->dstDisplayBounds.right-pWnd->dstDisplayBounds.left,	// GLsizei width,
						pWnd->dstDisplayBounds.bottom-pWnd->dstDisplayBounds.top,	//GLsizei height,
						GL_YCBCR_422_APPLE,				//GL_BGR_EXT,  //GL_BGRA_EXT, // GLenum format,
						GL_UNSIGNED_SHORT_8_8_REV_APPLE,//GL_BYTE,  //GL_UNSIGNED_INT_8_8_8_8_REV, //GLenum type
						pData); 	//const GLvoid *pixels
	}
}

void
videoTimer(VideoWindow* pWnd)
{
	OSErr   err;
	int		isUpdated = 0;

	// Stop the timer is we're not grabbing
	if (!vdgIsGrabbing( pWnd->pVdg))
		return;

	if (err = vdgIdle( pWnd->pVdg, &isUpdated))
	{
		printError("vdgIdle err=%d\n", err);
		goto endFunc;
	}

	if (isUpdated)
	{
		// TODO synchronization of texture update
		videoUpdateFrame(pWnd, pWnd->pDstData, pWnd->dstDataSize);
		glutPostRedisplay();
	}

endFunc:
	glutTimerFunc(pWnd->milliSecPerTimer, videoTimerDispatch, pWnd->winID);
	return;
}


// This function may be called for a window that has been destroyed.
void 
videoTimerDispatch(int winID)
{
	VideoWindow* pWnd;
	
	if (pWnd = (VideoWindow*) getDataForID(winID))
	{
		glutSetWindow(winID);
		videoTimer(pWnd);
	} else
		printError("videoTimerDispatch: unknown winID: %d\n", winID);
}


void 
videoKeyboard(VideoWindow* pWnd, unsigned char key, int x, int y )  
{
  switch ( key ) {
	
	//case 'c':		// New camera window
	case 'C':  
		videoNewWindow();
	break;
	case 'x':		// Destroy current window
	case 'X':
		videoDestroyWindow(pWnd);
	break;
	case 'f':		// Change filtering mode
	case 'F':
		if (pWnd->texFilter == GL_NEAREST)
		{
			pWnd->texFilter = GL_LINEAR;
		} else
		{
			pWnd->texFilter = GL_NEAREST;
		}	
		pWnd->texIsFirstUploaded = 0;
	break;
	case '0':
		if (pWnd->isFullScreen)
		{
			glutReshapeWindow( pWnd->widthLast, pWnd->heightLast);
			glutPositionWindow( pWnd->xPosLast, pWnd->yPosLast);
			pWnd->isFullScreen = 0;
		} else
		{
			pWnd->widthLast = glutGet(GLUT_WINDOW_WIDTH);
			pWnd->heightLast = glutGet(GLUT_WINDOW_HEIGHT);
			pWnd->xPosLast = glutGet(GLUT_WINDOW_X);
			pWnd->yPosLast = glutGet(GLUT_WINDOW_Y);
			pWnd->isFullScreen = 1;
			glutFullScreen();
		}
	break;
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	{
		int factor = key - '0';
		float scale;
		
		if (GLUT_ACTIVE_ALT & glutGetModifiers())
			scale = 1.0F / factor;
		else
			scale = factor;
		
		glutReshapeWindow(  scale * (pWnd->dstDisplayBounds.right - pWnd->dstDisplayBounds.left),
							scale * (pWnd->dstDisplayBounds.bottom - pWnd->dstDisplayBounds.top) );
	}
	break;
	default: 
		vdgSetKey(key, glutGetModifiers());
      break;
  }
}

void 
videoKeyboardDispatch ( unsigned char key, int x, int y )  
{
	int winID;
	VideoWindow* pWnd;
	
	winID = glutGetWindow();
	if (pWnd = (VideoWindow*) getDataForID(winID))
		videoKeyboard(pWnd, key, x, y);
	else
		printError("videoKeyboardDispatch: unknown winID");
}

void
videoMouse(VideoWindow* pWnd, int button, int state, int x, int y)
{
	switch (button)
	{
	case GLUT_LEFT_BUTTON:
		switch(state)
		{
		case GLUT_DOWN:
			printError("Left Mouse click down at %d, %d\n", x, y);
			vdgSetClick(0, x, y, glutGetModifiers());
			break;
		case GLUT_UP:
			printError("Left Mouse click up at %d, %d\n", x, y);
			vdgSetClick(1, x, y, glutGetModifiers());
			break;
		}
		break;
	
	case GLUT_RIGHT_BUTTON: 
		switch(state)
		{
		case GLUT_DOWN:
			vdgSetClick(2, x, y, glutGetModifiers());
			break;
		}
		break;
	}
}

void 
videoMouseDispatch ( int button, int state, int x, int y )  
{
	int winID;
	VideoWindow* pWnd;
	
	winID = glutGetWindow();
	if (pWnd = (VideoWindow*) getDataForID(winID))
		videoMouse(pWnd, button, state, x, y);
	else
		printError("videoMouseDispatch: unknown winID");
}

void
videoDestroyWindow(VideoWindow* pWnd)
{
	if (pWnd->texID)
	{
		if (pWnd->useTextureRange)
			glTextureRangeAPPLE(pWnd->texEnum, 0, NULL);

		glDeleteTextures(1, &pWnd->texID);
		pWnd->texID = 0;
	}

	if (pWnd->dstPort)
	{
		disposeOffscreenGWorld(pWnd->dstPort);
		pWnd->dstPort = NULL;
	}

	if (pWnd->vdImageDesc)
	{
		DisposeHandle((Handle)pWnd->vdImageDesc);
		pWnd->vdImageDesc = NULL;
	}

	if (pWnd->pVdg)
	{
		vdgStopGrabbing(pWnd->pVdg);	
		vdgUninit(pWnd->pVdg);
		vdgDelete(pWnd->pVdg);
		pWnd->pVdg = NULL;
	}	

	removeDataforID(pWnd->winID);

	glutDestroyWindow(pWnd->winID);
	
	free(pWnd);
}


OSErr
videoSetWindowTitle (VideoWindow* pWnd)
{
	OSErr err;

	long nameSize = 255;
	char szName[255];
	if (err = vdgGetDeviceNameAndFlags(pWnd->pVdg, szName, &nameSize, NULL))
	{
		printError("vdgGetDeviceName err=%d\n", err);
		sprintf(szName, "Video Window %d", pWnd->winID);
	}
	
	glutSetWindowTitle(szName);		
	
	return err;
}


OSErr
videoSetTimerPeriodFromVdig(VideoWindow* pWnd)
{
	OSErr err;

	// ASC soft vdig fails this call
	if (err = vdgGetDataRate(   pWnd->pVdg, 
								&pWnd->milliSecPerFrame,
								&pWnd->frameRate,
								&pWnd->bytesPerSecond))
	{
		printError("vdgGetDataRate err=%d\n", err);
//		goto endFunc; 
	}

	if (err == noErr)
	{
		// Some vdigs do return the frameRate but not the milliSecPerFrame
		if ((pWnd->milliSecPerFrame == 0) && (pWnd->frameRate != 0))
		{
			pWnd->milliSecPerFrame = (1000L << 16) / pWnd->frameRate;
		} 
	}
	
	// Poll the vdig at twice the frame rate or between sensible limits
	pWnd->milliSecPerTimer = pWnd->milliSecPerFrame / 2;
	if (pWnd->milliSecPerTimer <= 0)
	{
		printError("pWnd->milliSecPerFrame: %d ", pWnd->milliSecPerFrame);
		pWnd->milliSecPerTimer = gMilliSecPerTimerMin;
		printError("forcing timer period to %dms\n", pWnd->milliSecPerTimer);
	}
	if (pWnd->milliSecPerTimer >= gMilliSecPerTimerMax)
	{
		printError("pWnd->milliSecPerFrame: %d ", pWnd->milliSecPerFrame);
		pWnd->milliSecPerFrame = gMilliSecPerTimerMax;
		printError("forcing timer period to %dms\n", pWnd->milliSecPerTimer);
	}

	return err;
}

void
videoSetDisplayBoundsToCaptureBounds(VideoWindow* pWnd)
{
	// TODO offset from port bounds?
	pWnd->dstDisplayBounds.left = 0;
	pWnd->dstDisplayBounds.right = (*pWnd->vdImageDesc)->width;
	pWnd->dstDisplayBounds.top = 0;
	pWnd->dstDisplayBounds.bottom = (*pWnd->vdImageDesc)->height;		
}

int
videoGetGLUnpackAlignment(VideoWindow* pWnd)
{
	int depth = (*GetGWorldPixMap(pWnd->dstPort))->pixelSize / 8; // size of a pixel in bytes e.g. 3 for RGB, 2 for Y'CbCr 
	return ceilpow2i(GetPixRowBytes(GetGWorldPixMap(pWnd->dstPort)) - depth * (pWnd->dstPortBounds.right-pWnd->dstPortBounds.left));
}

int 
videoGetGLUnpackRowLength(VideoWindow* pWnd)
{
	return pWnd->dstPortBounds.right-pWnd->dstPortBounds.left;
}

void
videoGetTexCoords(VideoWindow* pWnd, float* aCoords)
{
	if (pWnd->texEnum == GL_TEXTURE_RECTANGLE_EXT)
	{
		aCoords[0] = pWnd->dstDisplayBounds.left;
		aCoords[1] = pWnd->dstDisplayBounds.right;
		aCoords[2] = pWnd->dstDisplayBounds.top;
		aCoords[3] = pWnd->dstDisplayBounds.bottom;
	} else
	{
		const float rTex_width = 1.0F / pWnd->texWidth;
		const float rTex_height = 1.0F / pWnd->texHeight;
		aCoords[0] = rTex_width * pWnd->dstDisplayBounds.left ;
		aCoords[1] = rTex_width * pWnd->dstDisplayBounds.right;
		aCoords[2] = rTex_height * pWnd->dstDisplayBounds.top;
		aCoords[3] = rTex_height * pWnd->dstDisplayBounds.bottom;
	}
}


